@@ -1,3 +1,4 @@
+import base64
 import json
 import os
 import sys
@@ -7,6 +8,7 @@ import urllib.request
 from os import PathLike
 from pathlib import Path
 from typing import List, Optional
+from uuid import uuid4
 from warnings import warn
 
 from modflow_devtools.zip import MFZipFile
@@ -128,11 +130,11 @@ def get_release(repo, tag="latest", quiet=False) -> dict:
     return release
 
 
-def get_release_assets(repo, tag="latest", quiet=False) -> List[dict]:
+def list_release_assets(repo, tag="latest", quiet=False) -> List[dict]:
     pass
 
 
-def get_artifacts(
+def list_artifacts(
     repo, name=None, per_page=None, max_pages=None, quiet=False
 ) -> List[dict]:
     """
@@ -198,20 +200,53 @@ def get_artifacts(
     return artifacts
 
 
+def download_artifact(
+    repo, id, path: Optional[PathLike] = None, delete_zip=True, quiet=False
+):
+    url = f"https://api.github.com/repos/{repo}/actions/artifacts/{id}/zip"
+    request = urllib.request.Request(url)
+    if "github.com" in url:
+        github_token = os.environ.get("GITHUB_TOKEN", None)
+        if github_token:
+            request.add_header("Authorization", f"Bearer {github_token}")
+
+    zip_path = Path(path).expanduser().absolute() / f"{str(uuid4())}.zip"
+    with urllib.request.urlopen(request) as url_file, open(
+        zip_path, "ab"
+    ) as out_file:
+        content = url_file.read()
+        out_file.write(content)
+
+    if not quiet:
+        print(f"Uncompressing: {zip_path}")
+
+    z = MFZipFile(zip_path)
+    z.extractall(str(path))
+    z.close()
+
+    # delete the zipfile
+    if delete_zip:
+        if not quiet:
+            print(f"Deleting zipfile {zip_path}")
+        zip_path.unlink()
+
+
 def download_and_unzip(
     url: str,
     path: Optional[PathLike] = None,
     delete_zip=True,
     verbose=False,
 ):
-    """Download and unzip a zip file from a url.
+    """
+    Download and unzip a zip file from a URL.
+    The filename must be the last element in the URL.
 
     Parameters
     ----------
     url : str
         url address for the zip file
     path : PathLike
-        path where the zip file will be saved (default is the current path)
+        path where the zip file will be saved (default is current path)
     delete_zip : bool
         boolean indicating if the zip file should be deleted after it is
         unzipped (default is True)
@@ -241,32 +276,36 @@ def download_and_unzip(
 
     # download zip file
     file_path = path / url.split("/")[-1]
-
+    request = urllib.request.Request(url)
     if "github.com" in url:
         github_token = os.environ.get("GITHUB_TOKEN", None)
         if github_token:
-            opener = urllib.request.build_opener()
-            opener.addheaders = [("Bearer", github_token)]
-            urllib.request.install_opener(opener)
+            request.add_header("Authorization", f"Bearer {github_token}")
 
-    _, headers = urllib.request.urlretrieve(
-        url, filename=str(file_path), reporthook=report
-    )
+    with urllib.request.urlopen(request) as url_file, open(
+        file_path, "ab"
+    ) as out_file:
+        content = url_file.read()
+        out_file.write(content)
 
-    # get content length, if available
-    tag = "Content-length"
-    if tag in headers:
-        file_size = headers[tag]
-        len_file_size = len(file_size)
-        file_size = int(file_size)
+        # _, headers = urllib.request.urlretrieve(
+        #     url, filename=str(file_path), reporthook=report
+        # )
 
-        bfmt = "{:" + f"{len_file_size}" + ",d}"
-        sbfmt = "{:>" + f"{len(bfmt.format(int(file_size)))}" + "s} bytes"
-        msg = f"   file size: {sbfmt.format(bfmt.format(int(file_size)))}"
-        if verbose:
-            print(msg)
-    else:
-        file_size = 0.0
+        # get content length, if available
+        tag = "Content-length"
+        if tag in url_file.headers:
+            file_size = url_file.headers[tag]
+            len_file_size = len(file_size)
+            file_size = int(file_size)
+
+            bfmt = "{:" + f"{len_file_size}" + ",d}"
+            sbfmt = "{:>" + f"{len(bfmt.format(int(file_size)))}" + "s} bytes"
+            msg = f"   file size: {sbfmt.format(bfmt.format(int(file_size)))}"
+            if verbose:
+                print(msg)
+        else:
+            file_size = 0.0
 
     # write the total download time
     toc = timeit.default_timer()
@@ -287,16 +326,22 @@ def download_and_unzip(
             p = "Could not unzip the file. Stopping."
             raise Exception(p)
         z.close()
+
+        # delete the zipfile
+        if delete_zip:
+            if verbose:
+                print(f"Deleting zipfile {file_path}")
+            file_path.unlink()
     elif "tar" in file_path.suffix:
         ar = tarfile.open(file_path)
         ar.extractall(path=str(path))
         ar.close()
 
-    # delete the zipfile
-    if delete_zip:
-        if verbose:
-            print(f"Deleting zipfile {file_path}")
-        file_path.unlink()
+        # delete the zipfile
+        if delete_zip:
+            if verbose:
+                print(f"Deleting zipfile {file_path}")
+            file_path.unlink()
 
     if verbose:
         print(f"Done downloading and extracting {file_path.name} to {path}")
